""" Some tests need appropriate API-KEY for the OpenAI API.

"""

import os
import sys
import io
import pytest
import aici
import pyperclip
from  aici import __version__

class Testaici:

    def test_keysetting(self):
        assert os.getenv('OPENAI_API_KEY') is not None, "OPENAI_API_KEY is not set"

    def test_version(self, monkeypatch, capsys):
        test_args = ['aici', '-v']
        monkeypatch.setattr(sys, 'argv', test_args)

        # Catch the SystemExit to prevent the test from failing
        with pytest.raises(SystemExit) as pytest_wrapped_e:
            aici.main()

        # Ensure the SystemExit code is 0
        assert pytest_wrapped_e.type == SystemExit
        assert pytest_wrapped_e.value.code == 0

        # Capture the output and check if the version was printed
        captured = capsys.readouterr()
        assert __version__ in captured.out

    def test_query(self, monkeypatch):
        # REAL CALL to API
        test_args = ['aici', 'Hello']
        monkeypatch.setattr(sys, 'argv', test_args)
        try:
            aici.main()
        except Exception as e:
            pytest.fail(f"command error: {e}")
            
    def test_query_with_model(self, monkeypatch):
        # REAL CALL to API
        test_args = ['aici', 'Hello', '--model=gpt-4o-mini-2024-07-18']
        monkeypatch.setattr(sys, 'argv', test_args)
        try:
            aici.main()
        except Exception as e:
            pytest.fail(f"command error: {e}")
            
    def test_query_with_system(self, monkeypatch):
        # REAL CALL to API
        test_args = ['aici', 'Hello', '--system="answer in Japnese Romaji only"']
        monkeypatch.setattr(sys, 'argv', test_args)
        try:
            aici.main()
        except Exception as e:
            pytest.fail(f"command error: {e}")
            
    def test_option_complete(self, monkeypatch):
        # REAL CALL to API
        test_args = ['aici', 'Hello', '--complete']
        monkeypatch.setattr(sys, 'argv', test_args)
        try:
            aici.main()
        except Exception as e:
            pytest.fail(f"command error: {e}")
            
    def test_option_clipboard(self, monkeypatch):
        # REAL CALL to API
        test_args = ['aici', 'Hello', '--output=clip']
        monkeypatch.setattr(sys, 'argv', test_args)
        try:
            aici.main()
        except Exception as e:
            pytest.fail(f"command error: {e}")
            
    def test_option_ai_out(self, monkeypatch):
        # REAL CALL to API
        # clipboard output
        test_args = ['aici', 'Hello', '--output=clip', '--system="すべてローマ字で答える"']
        monkeypatch.setattr(sys, 'argv', test_args)

        # to mock sys.stdout, using StringIO 
        mock_stdout = io.StringIO()
        monkeypatch.setattr(sys, 'stdout', mock_stdout)

        try:
            aici.main()
            clipboard_output = pyperclip.paste()

            assert all(ord(char) < 128 for char in clipboard_output), \
                f"Output contains non-ASCII characters: CONTENT:<<{clipboard_output}>>"

        except Exception as e:
            pytest.fail(f"command error: {e}")
            



"""
aici  CLI for AI

This module is designed in the form of a CLI to make it easier to invoke AI API calls from other tools.

Functions:
    def query_chatgpt(prompt:str, complete:bool=False, model:str=DEFAULT_MODEL, 
                  system:str=DEFAULT_SYSTEM, output=sys.stdout) -> None:

    def main() -> None:

Examples:
    >>> sys.argv = ['/path/to/aici', 'Hello']
    >>> aici.main()
    Hello! How can I assist you today


"""

import os
import io
import sys
import argparse
import openai
from openai import OpenAI
import pyperclip
import logging
from . import __version__

logging.basicConfig(level=logging.WARNING)
logger = logging.getLogger('aici')

# Initialize the OpenAI client with your API key
client = OpenAI(api_key=os.getenv('OPENAI_API_KEY'))

DEFAULT_MODEL = os.getenv("OPENAI_CHATGPT_MODEL", "gpt-4o")
DEFAULT_SYSTEM = os.getenv("OPENAI_CHATGPT_SYSTEM", "You are a helpful assistant.")

def query_chatgpt(prompt:str, complete:bool=False, model:str=DEFAULT_MODEL, 
                  system:str=DEFAULT_SYSTEM, output=sys.stdout) -> None:
    """  Sends a prompt to the OpenAI ChatGPT API and handles the response, either streaming or complete.

    Args:
        prompt (str): The prompt to send to the ChatGPT API.
        complete (bool): If False, the response will be streamed; if True, the complete response will be retrieved at once. Default is False.
        model (str): The model name to use for the API call. Default is set to the module's DEFAULT_MODEL.
        system (str): The system message to send as context to the API. Default is set to the module's DEFAULT_SYSTEM.
        output (file-like object): The output stream where the response will be written. Default is sys.stdout.

    Returns:
        None: This function doesn't return a value, but it prints the API response to the specified output stream.

    Raises:
        openai.APIConnectionError: If the server could not be reached.
        openai.RateLimitError: If the API rate limit is exceeded (429 status code).
        openai.APIStatusError: If any other non-200-range status code is received.
    """

    try:
        if complete == False:
            # Streaming response from OpenAI's API
            stream = client.chat.completions.create(
                model=model,
                messages=[
                    {"role": "system", "content": system},
                    {"role": "user", "content": prompt}
                ],
                stream=True  # Enable streaming mode
            )
        
            # Collecting and printing the streamed response
            collected_response = ""
            for chunk in stream:
                chunk_message = chunk.choices[0].delta.content or ""
                print(chunk_message, end="", flush=True, file=output)
                collected_response += chunk_message
        
            print() # Print a newline at the end
            return collected_response
    
        else:
            response = client.chat.completions.create(
                model=model,
                messages=[
                    {"role": "system", "content": system},
                    {"role": "user", "content": prompt}
                ]
            )
            # Print a newline at the end
            print(response.choices[0].message.content, flush=True, file=output)

    except openai.APIConnectionError as e:
        logger.error("The server could not be reached", exc_info=e)
        print(e.__cause__)  # an underlying Exception, likely raised within httpx.
    except openai.RateLimitError as e:
        logger.error("A 429 status code was received; we should back off a bit.", exc_info=e)
    except openai.APIStatusError as e:
        logger.error("Another non-200-range status code was received", exc_info=e)
        print(e.status_code)
        print(e.response)        
        
def main() -> None:

    try:
        parser = argparse.ArgumentParser(description="Query OpenAI's ChatGPT")
        parser.add_argument('prompt', type=str, nargs='?', default=argparse.SUPPRESS, help='The prompt to send to ChatGPT or "-" to read from stdin')
        parser.add_argument('-v', '--version', action='store_true', help='Show version and exit')
        parser.add_argument('-m', '--model', default=DEFAULT_MODEL, help='model name')
        parser.add_argument('-c', '--complete', default=False, action='store_true', help='get a message when completed')
        parser.add_argument('-s', '--system', default=DEFAULT_SYSTEM, help='spcify a system content')
        parser.add_argument('-o', '--output', help='output destination, "clip" for clipboard', default=sys.stdout)
        args = parser.parse_args()

        if args.version:
            print(__version__)
            sys.exit(0)
            # Check if 'prompt' exists and is not None

        if args.prompt is None:
            parser.error("the following arguments are required: prompt")
    
        # Check if the prompt is "-" and read from stdin if so
        if args.prompt == "-":
            prompt = sys.stdin.read().strip()
        else:
            prompt = args.prompt

        if args.output == 'clip' or args.output == 'clipboard':
            buffer = io.StringIO()
        else:
            buffer = sys.stdout

        query_chatgpt(prompt, model=args.model, complete=args.complete, system=args.system, output=buffer)

        if args.output == 'clip' or args.output == 'clipboard':
            pyperclip.copy(buffer.getvalue())

    except Exception as e:
        logger.error(f"Error", exc_info=e)
        raise Exception(f"Error: {e}")

if __name__ == "__main__":
    main()

    



"""
aici  CLI for AI

this module is designed in the form of a CLI to make it easier to invoke AI API calls from other tools.

This file handles the execution when called with python -m aici.

`python -m aici <arg1> <arg2>`

"""
import aici
from .main import main
from argparse import ArgumentParser

main()





"""
aici  CLI for AI

This module is designed in the form of a CLI to make it easier to invoke AI API calls from other tools.

test 
    env only 
    env and configuration file
    configuration file only

"""
import os
import sys
from .version import __version__
from dotenv import load_dotenv

env_candidates = [os.path.expanduser('~/.config/aici/config'), # 1st priority
                  os.path.expanduser('~/.aici')] # 2nd priority
API_KEY = None
_API_KEY = os.environ.get('OPENAI_API_KEY') # preserve value before reading config file

ary = []
for fn in env_candidates:
    if os.path.exists(fn):
        ary.append(fn)

# check must env values are set or config file exists
if _API_KEY is None:
    if len(ary) == 0:
        raise RuntimeError(f"Env file not existing {env_candidates}")

if len(ary) > 0:
    ENV_FILE = ary[0]
    load_dotenv(ENV_FILE)
    API_KEY = os.environ.get('OPENAI_API_KEY')  # env value is prior to config file

if API_KEY is None and _API_KEY is None: # not specified in env file
    raise RuntimeError("API KEY info not found")

if API_KEY is None:
    API_KEY = _API_KEY

from .main import main






# setup.py
from setuptools import setup, find_packages
# Import the version
from aici.version import __version__

setup(
    name="aici",
    version=__version__,
    packages=find_packages(),
    description="A command line interface for ChatGPT",
    long_description=open('README.md').read(),    
    long_description_content_type='text/markdown',    
    url="https://github.com/tkykszk/aici/",
    license="MIT",
    include_package_data=True,
    install_requires=[
        # プロジェクトの依存関係をここに記述
        # 例: 'requests', 'numpy'
        'python-dotenv',
        'openai>1.40.1',
        'pytest',
    ],
    entry_points={
        'console_scripts': [
            'aici=aici.main:main',
        ],        
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',  # Python version
)
